import heapq
from hipster.error import HeapError
from hipster.heap import Heap


class MinHeap(Heap):
    def __init__(self):
        super().__init__()

    def peek(self):
        if len(self.heap) == 0:
            raise HeapError("Peeking into an empty heap")
        with self.read_lock:
            return heapq.nsmallest(1, self.heap)[0]

    def pop(self):
        if len(self.heap) == 0:
            raise HeapError("Popping off an empty heap")
        with self.write_lock:
            return heapq.heappop(self.heap)








import abc
import heapq
from readerwriterlock import rwlock

from hipster.error import HeapError


class Heap(abc.ABC):
    def __init__(self):
        self.heap = []
        lock = rwlock.RWLockFair()
        self.read_lock = lock.gen_rlock()
        self.write_lock = lock.gen_wlock()

    def push(self, item):
        with self.write_lock:
            heapq.heappush(self.heap, item)

    @abc.abstractmethod
    def pop(self):
        pass

    @abc.abstractmethod
    def peek(self):
        pass

    def remove(self, item):
        store = []
        found = False
        with self.write_lock:
            while len(self.heap) > 0:
                curr = self.heap.pop()
                if curr == item:
                    found = True
                    break
                else:
                    store.append(curr)
            if not found:
                raise HeapError("Item not in Heap")
            while len(store) > 0:
                heapq.heappush(self.heap, store.pop())

    def clear(self):
        with self.write_lock:
            del self.heap[:]

    def __len__(self):
        with self.read_lock:
            return len(self.heap)



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="hipster",
    version="3.0.2",
    description="A thread safe implementation of the MinHeap and MaxHeap",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/soumasish/hipster",
    author="Soumasish Goswami",
    author_email="soumasish@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=['readerwriterlock', ],
)
